@@ -1,14 +1,15 @@
 package HTML::TokeParser;
 
-# $Id: TokeParser.pm,v 2.26 2003/10/10 10:45:46 gisle Exp $
+# $Id: TokeParser.pm,v 2.31 2005/10/24 09:54:32 gisle Exp $
 
 require HTML::PullParser;
 @ISA=qw(HTML::PullParser);
-$VERSION = sprintf("%d.%02d", q$Revision: 2.26 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%02d", q$Revision: 2.31 $ =~ /(\d+)\.(\d+)/);
 
 use strict;
 use Carp ();
 use HTML::Entities qw(decode_entities);
+use HTML::Tagset ();
 
 my %ARGS =
 (
@@ -18,6 +19,9 @@ my %ARGS =
  process     => "'PI',token0,text",
  comment     => "'C',text",
  declaration => "'D',text",
+
+ # options
+ unbroken_text => 1,
 );
 
 
@@ -59,6 +63,23 @@ sub get_tag
 }
 
 
+sub _textify {
+    my($self, $token) = @_;
+    my $tag = $token->[1];
+    return undef unless exists $self->{textify}{$tag};
+
+    my $alt = $self->{textify}{$tag};
+    my $text;
+    if (ref($alt)) {
+	$text = &$alt(@$token);
+    } else {
+	$text = $token->[2]{$alt || "alt"};
+	$text = "[\U$tag]" unless defined $text;
+    }
+    return $text;
+}
+
+
 sub get_text
 {
     my $self = shift;
@@ -72,15 +93,7 @@ sub get_text
 	} elsif ($type =~ /^[SE]$/) {
 	    my $tag = $token->[1];
 	    if ($type eq "S") {
-		if (exists $self->{textify}{$tag}) {
-		    my $alt = $self->{textify}{$tag};
-		    my $text;
-		    if (ref($alt)) {
-			$text = &$alt(@$token);
-		    } else {
-			$text = $token->[2]{$alt || "alt"};
-			$text = "[\U$tag]" unless defined $text;
-		    }
+		if (defined(my $text = _textify($self, $token))) {
 		    push(@text, $text);
 		    next;
 		}
@@ -91,6 +104,8 @@ sub get_text
 		 $self->unget_token($token);
 		 last;
 	    }
+	    push(@text, " ")
+		if $tag eq "br" || !$HTML::Tagset::isPhraseMarkup{$token->[1]};
 	}
     }
     join("", @text);
@@ -105,6 +120,35 @@ sub get_trimmed_text
     $text;
 }
 
+sub get_phrase {
+    my $self = shift;
+    my @text;
+    while (my $token = $self->get_token) {
+	my $type = $token->[0];
+	if ($type eq "T") {
+	    my $text = $token->[1];
+	    decode_entities($text) unless $token->[2];
+	    push(@text, $text);
+	} elsif ($type =~ /^[SE]$/) {
+	    my $tag = $token->[1];
+	    if ($type eq "S") {
+		if (defined(my $text = _textify($self, $token))) {
+		    push(@text, $text);
+		    next;
+		}
+	    }
+	    if (!$HTML::Tagset::isPhraseMarkup{$tag}) {
+		$self->unget_token($token);
+		last;
+	    }
+	    push(@text, " ") if $tag eq "br";
+	}
+    }
+    my $text = join("", @text);
+    $text =~ s/^\s+//; $text =~ s/\s+$//; $text =~ s/\s+/ /g;
+    $text;
+}
+
 1;
 
 
@@ -146,8 +190,8 @@ object, or the complete document to be parsed.
 
 If the argument is a plain scalar, then it is taken as the name of a
 file to be opened and parsed.  If the file can't be opened for
-reading, then the constructor will return an undefined value and $!
-will tell you why it failed.
+reading, then the constructor will return C<undef> and $! will tell
+you why it failed.
 
 If the argument is a reference to a plain scalar, then this scalar is
 taken to be the literal document to parse.  The value of this
@@ -158,6 +202,24 @@ C<HTML::TokeParser> can read() from when it needs more data.  Typically
 it will be a filehandle of some kind.  The stream will be read() until
 EOF, but not closed.
 
+Note that the parsing result will likely not be valid if raw undecoded
+UTF-8 is used as a source.  When parsing UTF-8 encoded files turn
+on UTF-8 decoding:
+
+   open(my $fh, "<:utf8", "index.html") || die "Can't open 'index.html': $!";
+   my $p = HTML::TokeParser->new( $fh );
+   # ...
+
+If a $filename is passed to the constructor the file will be opened in
+raw mode and the parsing result will only be valid if its content is
+Latin-1 or pure ASCII.
+
+If parsing from an UTF-8 encoded string buffer decode it first:
+
+   utf8::decode($document);
+   my $p = HTML::TokeParser->new( \$document );
+   # ...
+
 =item $p->get_token
 
 This method will return the next I<token> found in the HTML document,
@@ -176,7 +238,7 @@ like this:
   ["PI", $token0, $text]
 
 where $attr is a hash reference, $attrseq is an array reference and
-the rest are plain scalars.  The L<HTML::Parser/Attrspec> explains the
+the rest are plain scalars.  The L<HTML::Parser/Argspec> explains the
 details.
 
 =item $p->unget_token( @tokens )
@@ -223,7 +285,7 @@ before the first of the specified tags found. For example:
 
 will return the text up to either a paragraph of linebreak element.
 
-The text might span tags that should be I<texified>.  This is
+The text might span tags that should be I<textified>.  This is
 controlled by the $p->{textify} attribute, which is a hash that
 defines how certain tags can be treated as text.  If the name of a
 start tag matches a key in this hash then this tag is converted to
@@ -249,6 +311,17 @@ Same as $p->get_text above, but will collapse any sequences of white
 space to a single space character.  Leading and trailing white space is
 removed.
 
+=item $p->get_phrase
+
+This will return all text found at the current position ignoring any
+phrasal-level tags.  Text is extracted until the first non
+phrasal-level tag.  Textification of tags is the same as for
+get_text().  This method will collapse white space in the same way as
+get_trimmed_text() does.
+
+The definition of <i>phrasal-level tags</i> is obtained from the
+HTML::Tagset module.
+
 =back
 
 =head1 EXAMPLES
@@ -281,7 +354,7 @@ L<HTML::PullParser>, L<HTML::Parser>
 
 =head1 COPYRIGHT
 
-Copyright 1998-2001 Gisle Aas.
+Copyright 1998-2005 Gisle Aas.
 
 This library is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
